﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.ComponentModel;
using System.Drawing;
using System.Data.SqlClient;
using System.IO;
using System.Xml;
using System.Data;
using System.Data.Linq;
using System.Threading;

using WellExpert.Data;
using WellExpert.Data.Extensions;
using WellExpert.Common;
using WellExpert.Common.Classes;
using WellExpert.Factory;

using Infragistics.Windows;
using Infragistics.Windows.Controls;
using Infragistics.Windows.Editors;
using Infragistics.Windows.DataPresenter;

namespace WellExpert.Materials_Viewer_WPFControl
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    public partial class Materials_Viewer : UserControl
    {
        #region Declarations
        #region Cell Widths
        const int Field_Packet_Cell_Width = 75;
        const int Field_Material_Cell_Width = 100;
        const int Field_Requested_Cell_Width = 150;
        const int Field_Fulfilled_Cell_Width = 75;
        #endregion
        #endregion

        #region Variables
        FieldLayout _FieldLayout;

        Field Field_Packet;
        Field Field_Material;
        Field Field_Requested;
        Field Field_Fulfilled;

        Style Style_Packet;
        Style Style_Material;
        Style Style_Requested;
        Style Style_Fulfilled;

        private static bool _staticInit = false;

        static ComboBoxItemsProvider ItemsProvider_productList = new ComboBoxItemsProvider();

        private Person _Person;
        private eoc.EOCMaterialDataTable _eocmdt;

        private EpisodeOfCare _EpisodeOfCare;
        public EpisodeOfCare EpisodeOfCare
        {
            get { return _EpisodeOfCare; }
            set { _EpisodeOfCare = value; }
        }

        public Guid PersonGUID
        {
            set
            {
                _Person = Factory<Person>.Find(value);
                LoadPerson(_Person);
            }
            get { return _Person.PersonID; }

        }


        #endregion

        #region Constructor

        public Materials_Viewer()
        {
            InitializeComponent();
        }

        public Materials_Viewer(EpisodeOfCare _EOC)
            : this()
        {
            _EpisodeOfCare = _EOC;

            InitializeComponent();

            LoadPersonMaterials();
        }
        #endregion

        #region Functions


        private void GetCustomizations(string FileName, XamDataGrid _XamDataGrid)
        {
            try
            {
                bool exists = false;
                string Folder = ".\\Settings";
                XmlDocument xDoc = new XmlDocument();

                if (!Directory.Exists(Folder)) Directory.CreateDirectory(Folder);
                if (!Directory.Exists(string.Format("{0}\\System", Folder))) Directory.CreateDirectory(string.Format("{0}\\System", Folder));
                if (!Directory.Exists(string.Format("{0}\\{1}", Folder, Process.LANID))) Directory.CreateDirectory(string.Format("{0}\\{1}", Folder, Process.LANID));
                if (File.Exists(string.Format("{0}\\{1}\\{2}.xml", Folder, Process.LANID, FileName)))
                {
                    xDoc.Load(string.Format("{0}\\{1}\\{2}.xml", Folder, Process.LANID, FileName));
                    exists = true;
                }
                else
                {
                    string sss = string.Format("{0}\\System\\{1}.xml", Folder, FileName);
                    if (File.Exists(string.Format("{0}\\System\\{1}.xml", Folder, FileName)))
                    {
                        xDoc.Load(string.Format("{0}\\System\\{1}.xml", Folder, FileName));
                        exists = true;
                    }
                }

                if (exists)
                {
                    StringWriter sw = new StringWriter();
                    XmlTextWriter xw = new XmlTextWriter(sw);
                    xDoc.WriteTo(xw);
                    string s = sw.ToString();
                    _XamDataGrid.LoadCustomizations(s);
                }
            }
            catch { }
        }

        private void SaveCustomizations(string FileName, XamDataGrid _XamDataGrid)
        {
            string Folder = ".\\Settings";
            XmlDocument xDoc = new XmlDocument();
            xDoc.LoadXml(_XamDataGrid.SaveCustomizations());

            if (!Directory.Exists(Folder)) Directory.CreateDirectory(Folder);
            if (!Directory.Exists(string.Format("{0}\\{1}", Folder, Process.LANID))) Directory.CreateDirectory(string.Format("{0}\\{1}", Folder, Process.LANID));

            xDoc.Save(string.Format("{0}\\{1}\\{2}.xml", Folder, Process.LANID, FileName));
        }

        private static void LoadStatic()
        {
            if (_staticInit) return;

            _staticInit = true;
        }

        public void OpenEvent(Object sender, OpenEventArgs e)
        {
            _Person = Factory<Person>.Find((e.Tag as Person).PersonID);
            LoadPerson(_Person);
        }

        public void LoadPerson(Person p)
        {
            this._Person = p;
            LoadPersonMaterials();
        }

        public void RefreshMaterials()
        {
            LoadPersonMaterials();
        }

        private void LoadPersonMaterials()
        {
            if (this.Dispatcher.Thread != Thread.CurrentThread)
            {
                this.Dispatcher.Invoke(new System.Windows.Forms.MethodInvoker(this.LoadPersonMaterials));
                return;
            }

            

            //Load the person's material data into the local data table.
            _eocmdt = _Person[Person.Index.PersonMaterial] as eoc.EOCMaterialDataTable;

            try
            {
                //We will use LINQ to join our EOC Material data to the cache and display the 
                //results in a grid.
                var materials = from eoc.EOCMaterialRow eocm in _eocmdt.AsEnumerable()
                                join dbo.MaterialRow mat in Meta.Get<dbo.MaterialDataTable>().AsEnumerable()
                                on eocm.MaterialID equals mat.MaterialGUID
                                join dbo.PacketRow pak in Meta.Get<dbo.PacketDataTable>().AsEnumerable()
                                on eocm.Field<Guid?>("PacketID") equals pak.PacketGUID
                                into tempPackets
                                from Packets in tempPackets.DefaultIfEmpty()
                                orderby eocm.RequestedDate descending,
                                        mat.Description
                                select new
                                {
                                    Packet = (Packets == null ? string.Empty : Packets.Field<String>("Description")),
                                    Material = mat.Field<String>("Description"),
                                    Requested = Convert.ToDateTime(eocm.Field<DateTime>("RequestedDate").ToShortDateString()),
                                    Fulfilled = (eocm.Field<DateTime?>("FulfillmentDate") == null ? string.Empty : eocm.Field<DateTime>("FulfillmentDate").ToShortDateString())
                                };

                xamDataGrid_Materials.DataItems.Clear();

                DefineFields();

                xamDataGrid_Materials.DataSource = materials;

                //if (materials.Count() > 0)
                //{
                //    foreach (var MaterialRow in materials)
                //    {
                //        xamDataGrid_Materials.DataItems.Add(MaterialRow);
                //    }
                //}


                //ugMV.DataSource = materials.ToList();
                //this.ugMV.DisplayLayout.Bands[0].Columns["Packet"].CellMultiLine = Infragistics.Win.DefaultableBoolean.True;
                //this.ugMV.DisplayLayout.Bands[0].Columns["Material"].CellMultiLine = Infragistics.Win.DefaultableBoolean.True;
                //ugMV.LoadSettings("MV.Materials");
                //this.ugMV.DisplayLayout.Bands[0].Columns["Fulfilled"].SortIndicator = Infragistics.Win.UltraWinGrid.SortIndicator.Disabled;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        private void DefineFields()
        {
            try
            {
                //#region Packet
                //Field_Packet = new Field("Packet");
                //Field_Packet.Label = "Packet";

                //Style_Packet = new System.Windows.Style(typeof(XamTextEditor));
                //Style_Packet.Setters.Add(new Setter(XamTextEditor.TextWrappingProperty, TextWrapping.NoWrap));
                //Field_Packet.Settings.EditorStyle = Style_Packet;
                //Field_Packet.Settings.EditorType = typeof(XamTextEditor);
                //Field_Packet.Settings.AllowEdit = false;
                //Field_Packet.Settings.LabelWidth = Field_Packet_Cell_Width;
                //Field_Packet.Settings.CellMinWidth = Field_Packet_Cell_Width;
                //Field_Packet.Settings.LabelTextWrapping = System.Windows.TextWrapping.NoWrap;
                //Field_Packet.Visibility = Visibility.Visible;
                //#endregion

                //#region Material
                //Field_Material = new Field("Material");
                //Field_Material.Label = "Material";

                //Style_Material = new System.Windows.Style(typeof(XamTextEditor));
                //Style_Material.Setters.Add(new Setter(XamTextEditor.TextWrappingProperty, TextWrapping.Wrap));
                //Field_Material.Settings.EditorStyle = Style_Material;
                //Field_Material.Settings.EditorType = typeof(XamTextEditor);
                //Field_Material.Settings.AllowEdit = false;
                //Field_Material.Settings.LabelWidth = Field_Material_Cell_Width;
                //Field_Material.Settings.CellMinWidth = Field_Material_Cell_Width;
                //Field_Material.Settings.LabelTextWrapping = System.Windows.TextWrapping.Wrap;
                //Field_Material.Visibility = Visibility.Visible;
                //#endregion

                //#region Requested
                //Field_Requested = new Field("Requested");
                //Field_Requested.Label = "Requested";

                //Style_Requested = new System.Windows.Style(typeof(XamTextEditor));
                //Style_Requested.Setters.Add(new Setter(XamTextEditor.TextWrappingProperty, TextWrapping.Wrap));
                //Field_Requested.Settings.EditorStyle = Style_Requested;
                //Field_Requested.Settings.EditorType = typeof(XamTextEditor);
                //Field_Requested.Settings.AllowEdit = false;
                //Field_Requested.Settings.LabelWidth = Field_Requested_Cell_Width;
                //Field_Requested.Settings.CellMinWidth = Field_Requested_Cell_Width;
                //Field_Requested.Settings.LabelTextWrapping = System.Windows.TextWrapping.Wrap;
                //Field_Requested.Visibility = Visibility.Visible;
                //#endregion

                //#region Fulfilled
                //Field_Fulfilled = new Field("Fulfilled");
                //Field_Fulfilled.Label = "Fulfilled";

                //Style_Fulfilled = new System.Windows.Style(typeof(XamTextEditor));
                //Style_Fulfilled.Setters.Add(new Setter(XamTextEditor.TextWrappingProperty, TextWrapping.Wrap));
                //Field_Fulfilled.Settings.EditorStyle = Style_Fulfilled;
                //Field_Fulfilled.Settings.EditorType = typeof(XamTextEditor);
                //Field_Fulfilled.Settings.AllowEdit = false;
                //Field_Fulfilled.Settings.LabelWidth = Field_Fulfilled_Cell_Width;
                //Field_Fulfilled.Settings.CellMinWidth = Field_Fulfilled_Cell_Width;
                //Field_Fulfilled.Settings.LabelTextWrapping = System.Windows.TextWrapping.Wrap;
                //Field_Fulfilled.Visibility = Visibility.Visible;
                //#endregion

                //#region Field Layout
                //_FieldLayout = new FieldLayout();

                //_FieldLayout.Fields.Add(Field_Packet);
                //_FieldLayout.Fields.Add(Field_Material);
                //_FieldLayout.Fields.Add(Field_Requested);
                //_FieldLayout.Fields.Add(Field_Fulfilled);
                //#endregion

                //xamDataGrid_Materials.FieldLayouts.Add(_FieldLayout);

            }
            catch { }
        }

        private void xamDataGrid_Materials_InitializeRecord(object sender, Infragistics.Windows.DataPresenter.Events.InitializeRecordEventArgs e)
        {
            try
            {
                XamDataGrid g = (XamDataGrid)sender;
                switch (g.Name)
                {
                    case "xamDataGrid_Materials":
                    case "xamDataGrid_MaterialsViewer":
                        this.GetCustomizations("WPFMaterialsViewerGridCustomizations", g);
                        break;
                }
            }
            catch { }
        }

        private void xamDataGrid_Materials_FieldLayoutInitialized(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e)
        {
            try
            {
                XamDataGrid g = (XamDataGrid)sender;
                switch (g.Name)
                {
                    case "xamDataGrid_Materials":
                    case "xamDataGrid_MaterialsViewer":
                        this.GetCustomizations("WPFMaterialsViewerGridCustomizations", g);
                        break;
                }
            }
            catch { }
        }

        private void xamDataGrid_Materials_FieldPositionChanged(object sender, Infragistics.Windows.DataPresenter.Events.FieldPositionChangedEventArgs e)
        {
            try
            {
                XamDataGrid g = (XamDataGrid)sender;
                switch (g.Name)
                {
                    case "xamDataGrid_Materials":
                    case "xamDataGrid_MaterialsViewer":
                        this.SaveCustomizations("WPFMaterialsViewerGridCustomizations", g);
                        break;
                }
            }
            catch { }
        }

        private void xamDataGrid_Materials_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private void xamDataGrid_Materials_LayoutUpdated(object sender, EventArgs e)
        {

        }
        #endregion
    }
}
